/************************************************************
	FILE: math.c

************************************************************/

#include "defs.h"
#include "globals.h"

/************************************************************
	PROC:	lin_eq_solv(A,b,x,n)
	DOES:	Solves a set of n linear equations
************************************************************/

lin_eq_solv(a,b,x,n)
MAT a;
/* float **a; */
float *b,*x;
int n;
{
	int indx[40];
	float d;
	int n1;
	int flag;

	flag = ludcmp(a,n,indx,&d);
	if (!flag) return(0);
	lubksb(a,n,indx,b);
	for (n1=0; n1<n; n1++)
		x[n1] = b[n1];
	return(1);
}

/************************************************************
	PROC:	ludcmp()
	DOES:	replaces a matrix by the LU decomposition of itself
************************************************************/

#define TINY 1.0e-20

ludcmp(a,n,indx,d)
int n,*indx;
MAT a;
/* float **a; */
float *d;
{
	register int i,j,k;
	register float sum;
	int imax;
	float big,dum,temp;
	float *vv,*vector();
	extern nrerror(),free_vector();

	vv = vector(1,n);
	*d = 1.0;
	for (i=0; i<n; i++) {
		big = 0.0;
		for (j=0; j<n; j++)
			if ((temp=fabs(a[i][j]))  > big) big=temp;
		if (big==0.0) {
			nrerror("Singular matrix in routine LUDCMP");
			return(0);
		}
		vv[i] = 1.0/big;
	}
	for (j=0; j<n; j++) {
		for (i=0; i<j; i++) {
			sum = a[i][j];
			for (k=0; k<i; k++) sum -= a[i][k]*a[k][j];
			a[i][j] = sum;
		}
		big = 0.0;
		for (i=j; i<n; i++) {
			sum= a[i][j];
			for (k=0; k<j; k++)
				sum -= a[i][k]*a[k][j];
			a[i][j] = sum;
			if ((dum=vv[i]*fabs(sum))>=big) {
				big = dum;
				imax = i;
			}
		}
		if (j!=imax) {
			for (k=0; k<n; k++) {
				dum = a[imax][k];
				a[imax][k] = a[j][k];
				a[j][k] = dum;
			}
			*d = -(*d);
			vv[imax] = vv[j];
		}
		indx[j] = imax;
		if (a[j][j]==0.0) a[j][j]=TINY;
		if (j!=n-1) {
			dum = 1.0/(a[j][j]);
			for (i=j+1; i<n; i++) a[i][j] *= dum;
		}
	}
	free_vector(vv,1,n);
	return(1);
}

/************************************************************
	PROC:	lubksb()
	DOES:	does back-substitution for LU decomp
************************************************************/

lubksb(a,n,indx,b)
MAT a;
/* float **a; */
float b[];
int n,*indx;
{
	int i,ii=-1,ip,j;
	float sum;

	for (i=0; i<n; i++) {
		ip=indx[i];
		sum=b[ip];
		b[ip]=b[i];
		if (ii!=-1)
			for (j=ii; j<=i-1; j++) sum -= a[i][j]*b[j];
		else if (sum) ii=i;
		b[i] = sum;
	}
	for (i=n-1; i>=0; i--) {
		sum=b[i];
		for (j=i+1; j<n; j++) sum -= a[i][j]*b[j];
		b[i] = sum/a[i][i];
	}
}

/************************************************************
	PROC:	vector()
	DOES:	allocates a vector
************************************************************/

float *vector(nl,nh)
int nl,nh;
{
	extern char *malloc();

	return((float *) malloc(nh*sizeof(float)));
}

/************************************************************
	PROC:	free_vector()
	DOES:	frees the space for a vector
************************************************************/

free_vector(v,nl,nh)
float *v;
int nl,nh;
{
	free((char *) v);
}

/************************************************************
	PROC:	nrerror(str)
	DOES:	numerical recipe error
************************************************************/

nrerror(str)
STR str;
{
	fprintf(stderr,"nrerror: %s\n",str);
	exit(-1);
}

