/************************************************************
	FILE: cm5.cpp

************************************************************/

#include "defs.h"
#include "globals.h"
#include <math.h>

/************************************************************
	PROC:	dp_header
	DOES:	prints out header for dynamics procedure
************************************************************/

void dp_header(FILE *file)
{
	fprintf(file,"#include <math.h>\n");
	fprintf(file,"#include \"dyn_otf.h\"\n");
	link_defs(file);			/* place in link definitions */
	fprintf(file,"dyn_%s(st,t,Fext,ac)\n",PRocname);
	fprintf(file,"float *st,*t,*Fext,*ac;\n");
	fprintf(file,"{\n");
	local_vars(file);			/* declare local vars required */
	const_calc(file);
	fprintf(file,"\n");
}

/************************************************************
	PROC:	dp_tail
	DOES:	generates tail end of dynamics procedure
************************************************************/

void dp_tail(FILE *file)
{
	int num;
	LINK *link;
	STR find;

		/* calculate results */
	fprintf(file,"\tlin_eq_solv(A,b,x,%d);\n",Msize);
		/* copy results back into ddth and arx, ary */
	num = find_var("arx");
	fprintf(file,"\tac[0] = x[%d];\n",num);
	num = find_var("ary");
	fprintf(file,"\tac[1] = x[%d];\n",num);
	for (link=Link; link; link=link->next) {
		if (!INverse)
			sprintf(find,"ddth%d",link->num);
		else
			sprintf(find,"t[%d]",link->num-1);
		num = find_var(find);
		fprintf(file,"\tac[%d] = x[%d];\n",link->num+1,num);
	}
		/* close procedure */
	fprintf(file,"}\n");
	fprintf(file,"\n");
}

/************************************************************
	PROC:	dp_body()
	DOES:	fills in A matrix and b vector
************************************************************/

void dp_body(FILE *file)
{
	int row,col;
	TERM *eqn;

		/* fill in A matrix */
	for (row=0; row<Msize; row++) {
		for (col=0; col<Msize; col++) {
			fprintf(file,"\tA[%d][%d] = ",row,col);
			eqn = Eqmat[row][col];
			type_eqn(file,eqn);
		}
	}
		/* fill in b vector */
	for (row=0; row<Msize; row++) {
		fprintf(file,"\tb[%d] = ",row);
		eqn = Eqconst[row];
		type_eqn(file,eqn);
	}
}

/************************************************************
	PROC:	gen_file
	DOES:	generates file with dynamics procedure
************************************************************/

void gen_file(FILE *file)
{
	dp_header(file);
	dp_body(file);
	dp_tail(file);
}

/************************************************************
	PROC:	find_var
	DOES:	returns number of unknown
************************************************************/

int find_var(STR var_name)
{
	int n;

	for (n=0; n<Msize; n++) {
		if (!strcmp(Var[n],var_name))
			return(n);
	}
	fprintf(stderr,"Var '%s' not found\n",var_name);
	exit(-1);
}

/************************************************************
	PROC:	local_vars(file)
	DOES:	declare necessary local variables
************************************************************/

void local_vars(FILE *file)
{
	int max_knum = 0;
	int knum;
	char *name;
	int n;

	fprintf(file,"\tfloat A[MAX_EQNS][MAX_EQNS],b[MAX_EQNS],x[MAX_EQNS];\n");
	for (n=0; n<Const_eqns.num_eqns; n++) {		/* for each const eqn */
		name = Const_eqns.eqn[n].label;
		if (!strncmp(name,"k[",2)) {			/* a k const */
			sscanf(name,"k[%d",&knum);
			if (knum>max_knum) max_knum=knum;
		} else {				     	/* another const */
			fprintf(file,"\tfloat %s;\n",name);
		}
	}
	fprintf(file,"\tfloat k[%d];\n",max_knum+1);
	fprintf(file,"\n");
}

/************************************************************
	PROC:	link_defs
	DOES:	generates def statements with link dimensions
************************************************************/

void link_defs(FILE *file)
{
	LINK *link;
	float r,theta;
	MON *mon;

//	fprintf(file,"#define MAX_EQNS 20\n");
	fprintf(file,"#define G (%6.3f)\n",GRavity);
	fprintf(file,"#define MSIZE %d\n",Msize);
	for (link=Link; link; link=link->next) {
		fprintf(file,"#define MASS%d %f\n",link->num,link->mass);
		fprintf(file,"#define INER%d %f\n",link->num,link->iner);
		rtheta(link->cmx,link->cmy,&r,&theta);
		fprintf(file,"#define LENCM%d %f\n",link->num,r);
		fprintf(file,"#define A_CM%d %f\n",link->num,theta);
		if (link->fix) {
			rtheta(link->pfx,link->pfy,&r,&theta);
			fprintf(file,"#define LENFIX%d %f\n",link->num,r);
			fprintf(file,"#define A_F%d %f\n",link->num,theta);
		}
		for (mon=link->mon; mon; mon=mon->next) {
			rtheta(mon->x,mon->y,&r,&theta);
			fprintf(file,"#define LENMON%d %f\n",mon->num,r);
			fprintf(file,"#define A_M%d %f\n",mon->num,theta);
		}
		rtheta(link->parx,link->pary,&r,&theta);
		fprintf(file,"#define LENAT%d %f\n",link->num,r);
		fprintf(file,"#define A_AT%d %f\n",link->num,theta);
	}
	fprintf(file,"\n");
}

/************************************************************
	PROC:	type_eqn
	DOES:	prints equation to a file
************************************************************/

void type_eqn(FILE *file, TERM *eqn)
{
	TERM *term;
	P *p;

	if (!eqn) {
		fprintf(file,"\t\t0;\n");
		return;
	}
	term = eqn;
	while(term) {
		if (term==eqn)
			fprintf(file,"\t\t%c",term->neg?'-':' ');
		else
			fprintf(file,"\t\t%c",term->neg?'-':'+');
		p = term->p;
		if (p) {
			fprintf(file,"%s",p->txt);
			p = p->next;
		} else fprintf(file,"1");
		while (p) {
			fprintf(file,"*%s",p->txt);
			p = p->next;
		}
		term = term->next;
		if (term)	fprintf(file,"\n");
		else fprintf(file,";\n");
	}			/* end while(term) */
}

/************************************************************
	PROC:	rtheta(x,y,r,theta)
	DOES:	rectangular to polar conversion
************************************************************/

void rtheta(float x, float y, float *r, float *theta)
{
	*r = (float) sqrt(x*x+y*y);
	if (*r < 1.0e-6) *theta=0.0;
	else *theta = (float) atan2(y,x);
}

/************************************************************
	PROC:	const_calc()
	DOES:	prints out all constant calculations
************************************************************/

void const_calc(FILE *file)
{
	int n;				/* eqn number */

	for (n=0; n<Const_eqns.num_eqns; n++) {		/* for each eqn */
		fprintf(file,"\t%s = ",Const_eqns.eqn[n].label);
		type_eqn(file,Const_eqns.eqn[n].terms);
	}
}



